import { BaseConfigurationStore } from '@fvlab/configurationstore';
import { PoolConfig } from 'pg';
/**
 * Configuration parameters for the PostgreSQL client.
 *
 * @export
 * @interface PostgreSqlConfiguration
 * @extends {PoolConfig}
 */
export interface PostgreSqlConfiguration extends PoolConfig {
    /**
     * Default database to use to connect to the PostgreSQL server to verify that the configuration table exists.
     *
     * @type {string}
     * @memberof PostgreSqlConfiguration
     */
    default_database?: string;
    /**
     * Table to use to store the configuration.
     *
     * @type {string}
     * @memberof PostgreSqlConfiguration
     */
    tableName?: string;
}
/**
 * PostgrSQL based configuration store.
 *
 * @export
 * @class PostgreSqlConfigurationStore
 * @extends {BaseConfigurationStore}
 */
export declare class PostgreSqlConfigurationStore extends BaseConfigurationStore {
    private pool;
    private tableName;
    private config;
    /**
     * Initiate PostgrSqlConfigurationStore.
     * If the database does not exist, it creates a new one.
     * @param {(PostgreSqlConfiguration)} [config] Configuration object or URL or ENV variables
     * @returns {Promise<this>} A promise of this instance of the configuration store.
     * @memberof PostgreSqlConfigurationStore
     */
    init(config?: PostgreSqlConfiguration): Promise<this>;
    /**
     * Save data in the DB at the given path.
     *
     * @template T
     * @param {string} settingsPath Path to save data at.
     * @param {T} value Value of the data to save.
     * @returns {Promise<T>} Value that was saved.
     * @memberof PostgreSqlConfigurationStore
     */
    protected setData<T>(settingsPath: string, value: T): Promise<T>;
    /**
     * Retreive's data at a given path.  If the data doesn't exist at that path it adds the default value provided.
     *
     * @template T
     * @param {string} settingsPath Path to get data from.
     * @param {(T | undefined)} [defaultValue] Default value to save/use if the requested path is empty.
     * @returns {(Promise<T | undefined>)} The requested data if it exists or default if it does not.
     * @memberof PostgreSqlConfigurationStore
     */
    protected getData<T>(settingsPath: string, defaultValue?: T | undefined): Promise<T | undefined>;
    /**
     * Update data at a given path. Note, it will overwrite the data.
     *
     * @template T
     * @param {string} settingsPath Path to update data at.
     * @param {T} value Value of the data to update.
     * @returns {Promise<T>} Updated data.
     * @memberof PostgreSqlConfigurationStore
     */
    protected updateData<T>(settingsPath: string, value: T): Promise<T>;
    /**
     * Connect to the PostgreSQL database. If a connection already exists, close it and open a new one.
     *
     * @private
     * @param {PostgreSqlConfiguration} [config]
     * @param {boolean} [usePool=true] Uses a global connection pool rather than a client
     * @returns {(Promise<PoolClient | Client>)}
     * @memberof PostgreSqlConfigurationStore
     */
    private connect;
    /**
     * Close the connection to the PostgreSQL database.
     *
     * @private
     * @param {(PoolClient | Client)} client
     * @returns {Promise<void>}
     * @memberof PostgreSqlConfigurationStore
     */
    private disconnect;
    /**
     * Determines if the client is a regular Client.
     *
     * For more on Type Guards
     * See: https://www.typescriptlang.org/docs/handbook/advanced-types.html#user-defined-type-guards
     *
     * @private
     * @param {(PoolClient | Client)} client Client to check.
     * @returns {client is Client} If the client is a Client.
     * @memberof PostgreSqlConfigurationStore
     */
    private isClient;
    /**
     * Determines if the client is a Pool Client.
     *
     * For more on Type Guards
     * See: https://www.typescriptlang.org/docs/handbook/advanced-types.html#user-defined-type-guards
     *
     * @private
     * @param {(PoolClient | Client)} client Client to check.
     * @returns {client is PoolClient} If the client is a Pool Client.
     * @memberof PostgreSqlConfigurationStore
     */
    private isPoolClient;
    /**
     * Wrap a function in a connection so that it will automatically open and close the connection to the DB.
     *
     * @private
     * @template R Return type of the wrapped function.
     * @param {((client?: PoolClient | Client) => Promise<R>)} func Function to wrap.
     * @param {PostgreSqlConfiguration} [config] Connection configuration, if it is different than the class configuration.
     * @returns {Promise<R>} The result of the wrapped function.
     * @memberof PostgreSqlConfigurationStore
     */
    private connectionWrapper;
    /**
     *Creates the initial database if there isn't one.
     *
     * @param {PostgreSqlConfiguration} [config] Configuration Override
     * @memberof PostgreSqlConfigurationStore
     */
    createDatabase(config?: PostgreSqlConfiguration): Promise<void>;
    /**
     * Creates the initial tables for the database that the client
     * is connected to.
     *
     * @param {PostgreSqlConfiguration} [config] Configuration Override
     * @memberof PostgresDatabase
     */
    createTables(config?: PostgreSqlConfiguration): Promise<void>;
}
