"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const configurationstore_1 = require("@fvlab/configurationstore");
const pg_1 = require("pg");
const format = require("pg-format");
const fs_extra_1 = require("fs-extra");
/**
 * Template for the configuration table.
 *
 * @class PostgresSqlConfigurationEntryTemplate
 * @implements {PostgresSqlConfigurationEntry<T>}
 * @template T
 */
class PostgresSqlConfigurationEntryTemplate {
    constructor(config_path, data, id) {
        this.config_path = config_path;
        this.data = data;
        this.id = id;
    }
}
/**
 * PostgrSQL based configuration store.
 *
 * @export
 * @class PostgreSqlConfigurationStore
 * @extends {BaseConfigurationStore}
 */
class PostgreSqlConfigurationStore extends configurationstore_1.BaseConfigurationStore {
    constructor() {
        super(...arguments);
        this.tableName = 'config';
    }
    /**
     * Initiate PostgrSqlConfigurationStore.
     * If the database does not exist, it creates a new one.
     * @param {(PostgreSqlConfiguration)} [config] Configuration object or URL or ENV variables
     * @returns {Promise<this>} A promise of this instance of the configuration store.
     * @memberof PostgreSqlConfigurationStore
     */
    async init(config) {
        if (!config) {
            // Read from JSON file.
            throw new Error('PostgreSQL configuration must be present');
        }
        this.config = config;
        const tempConfig = {
            host: config.host,
            port: config.port,
            database: config.default_database || 'postgres',
            user: config.user,
            password: config.password // Password to connect with.
        };
        await this.connectionWrapper(async (client) => {
            const checkDbExistsQuery = format("SELECT * FROM pg_database WHERE datname = '%I';", config.database.toLowerCase());
            const result = await client.query(checkDbExistsQuery);
            if (result.rowCount === 0) {
                // No Database exists
                await this.createDatabase(tempConfig); // If database is new then we need tables to be created.
            }
        }, tempConfig);
        await this.createTables();
        return this;
    }
    /**
     * Save data in the DB at the given path.
     *
     * @template T
     * @param {string} settingsPath Path to save data at.
     * @param {T} value Value of the data to save.
     * @returns {Promise<T>} Value that was saved.
     * @memberof PostgreSqlConfigurationStore
     */
    async setData(settingsPath, value) {
        const template = new PostgresSqlConfigurationEntryTemplate(settingsPath, value);
        return this.connectionWrapper(async (client) => {
            const sqlFirstParam = '$1';
            const sqlSecondParam = 'CAST($2 AS json)';
            const configPath = 'config_path';
            const uniqueKeyName = 'config_un';
            const sql = format(`INSERT INTO %1$I (%2$s) VALUES %3$s ON CONFLICT ON CONSTRAINT %5$I DO UPDATE SET data = %4$s;`, this.tableName, [configPath, 'data'], // IDs aren't needed for inserts
            [[sqlFirstParam, sqlSecondParam]], // Generate SQL params - The cast is required as there is no implicit cast from string to JSON
            sqlSecondParam, uniqueKeyName);
            const sqlParam = [template.config_path, JSON.stringify(template.data)];
            await client.query(sql, sqlParam);
            return value;
        });
    }
    /**
     * Retreive's data at a given path.  If the data doesn't exist at that path it adds the default value provided.
     *
     * @template T
     * @param {string} settingsPath Path to get data from.
     * @param {(T | undefined)} [defaultValue] Default value to save/use if the requested path is empty.
     * @returns {(Promise<T | undefined>)} The requested data if it exists or default if it does not.
     * @memberof PostgreSqlConfigurationStore
     */
    async getData(settingsPath, defaultValue) {
        return this.connectionWrapper(async (client) => {
            let sql = `SELECT * FROM ${this.tableName}`;
            const sqlParam = [];
            if (settingsPath) {
                sql += ' WHERE config_path = $1';
                sqlParam.push(settingsPath);
            }
            sql += ' LIMIT 1';
            sql += ';';
            const queryResult = await client.query(sql, sqlParam);
            if (queryResult.rowCount) {
                const queryResultEntry = queryResult.rows[0];
                return queryResultEntry.data;
            }
            // Set default value in DB
            else
                return await this.setData(settingsPath, defaultValue);
        });
    }
    /**
     * Update data at a given path. Note, it will overwrite the data.
     *
     * @template T
     * @param {string} settingsPath Path to update data at.
     * @param {T} value Value of the data to update.
     * @returns {Promise<T>} Updated data.
     * @memberof PostgreSqlConfigurationStore
     */
    async updateData(settingsPath, value) {
        return this.setData(settingsPath, value);
    }
    /**
     * Connect to the PostgreSQL database. If a connection already exists, close it and open a new one.
     *
     * @private
     * @param {PostgreSqlConfiguration} [config]
     * @param {boolean} [usePool=true] Uses a global connection pool rather than a client
     * @returns {(Promise<PoolClient | Client>)}
     * @memberof PostgreSqlConfigurationStore
     */
    async connect(config, usePool = true) {
        const sqlConfig = config || this.config;
        if (!this.pool && usePool) {
            this.pool = new pg_1.Pool(sqlConfig);
        }
        const client = !usePool ? new pg_1.Client(sqlConfig) : await this.pool.connect();
        if (!usePool)
            client.connect();
        return client;
    }
    /**
     * Close the connection to the PostgreSQL database.
     *
     * @private
     * @param {(PoolClient | Client)} client
     * @returns {Promise<void>}
     * @memberof PostgreSqlConfigurationStore
     */
    async disconnect(client) {
        if (this.isPoolClient(client))
            await client.release();
        else if (this.isClient(client))
            await client.end();
    }
    /**
     * Determines if the client is a regular Client.
     *
     * For more on Type Guards
     * See: https://www.typescriptlang.org/docs/handbook/advanced-types.html#user-defined-type-guards
     *
     * @private
     * @param {(PoolClient | Client)} client Client to check.
     * @returns {client is Client} If the client is a Client.
     * @memberof PostgreSqlConfigurationStore
     */
    isClient(client) {
        return client && 'end' in client;
    }
    /**
     * Determines if the client is a Pool Client.
     *
     * For more on Type Guards
     * See: https://www.typescriptlang.org/docs/handbook/advanced-types.html#user-defined-type-guards
     *
     * @private
     * @param {(PoolClient | Client)} client Client to check.
     * @returns {client is PoolClient} If the client is a Pool Client.
     * @memberof PostgreSqlConfigurationStore
     */
    isPoolClient(client) {
        return client && 'release' in client;
    }
    /**
     * Wrap a function in a connection so that it will automatically open and close the connection to the DB.
     *
     * @private
     * @template R Return type of the wrapped function.
     * @param {((client?: PoolClient | Client) => Promise<R>)} func Function to wrap.
     * @param {PostgreSqlConfiguration} [config] Connection configuration, if it is different than the class configuration.
     * @returns {Promise<R>} The result of the wrapped function.
     * @memberof PostgreSqlConfigurationStore
     */
    async connectionWrapper(func, config) {
        const client = await this.connect(config, !config);
        const returnValue = await func(client);
        await this.disconnect(client);
        return returnValue;
    }
    /**
     *Creates the initial database if there isn't one.
     *
     * @param {PostgreSqlConfiguration} [config] Configuration Override
     * @memberof PostgreSqlConfigurationStore
     */
    async createDatabase(config) {
        await this.connectionWrapper(async (client) => {
            const createDbQuery = format('CREATE DATABASE %I TEMPLATE template0;', this.config.database);
            console.log('Creating Config Database', 'createDbQuery:', createDbQuery);
            try {
                await client.query(createDbQuery);
            }
            catch (e) {
                console.warn('createDatabase', 'database creation failed:', e.message);
            }
        }, config);
    }
    /**
     * Creates the initial tables for the database that the client
     * is connected to.
     *
     * @param {PostgreSqlConfiguration} [config] Configuration Override
     * @memberof PostgresDatabase
     */
    async createTables(config) {
        await this.connectionWrapper(async (client) => {
            const databaseQuery = (await fs_extra_1.readFile(`${__dirname}/sql/init_config.sql`)).toString();
            await client.query(databaseQuery);
        }, config);
    }
}
exports.PostgreSqlConfigurationStore = PostgreSqlConfigurationStore;
//# sourceMappingURL=index.js.map